package flink.zanxiangnet.ad.monitoring;

import com.tencent.ads.model.HourlyReportsGetListStruct;
import flink.zanxiangnet.ad.monitoring.pojo.AdInfo;
import flink.zanxiangnet.ad.monitoring.pojo.PlanInfo;
import flink.zanxiangnet.ad.monitoring.pojo.StatInfo;
import flink.zanxiangnet.ad.monitoring.pojo.dto.AdDataOfHourDTO;
import flink.zanxiangnet.ad.monitoring.util.DateUtil;
import flink.zanxiangnet.ad.monitoring.util.JsonUtil;
import flink.zanxiangnet.ad.monitoring.util.NumberUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.eventtime.SerializableTimestampAssigner;
import org.apache.flink.api.common.eventtime.WatermarkStrategy;
import org.apache.flink.api.common.functions.ReduceFunction;
import org.apache.flink.api.common.serialization.SimpleStringSchema;
import org.apache.flink.api.java.functions.KeySelector;
import org.apache.flink.connector.kafka.source.KafkaSource;
import org.apache.flink.connector.kafka.source.enumerator.initializer.OffsetsInitializer;
import org.apache.flink.streaming.api.datastream.DataStreamSource;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
import org.apache.flink.streaming.api.functions.sink.SinkFunction;
import org.apache.flink.streaming.api.functions.windowing.WindowFunction;
import org.apache.flink.streaming.api.windowing.assigners.TumblingEventTimeWindows;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.apache.kafka.clients.CommonClientConfigs;
import org.apache.kafka.common.config.SaslConfigs;
import org.apache.kafka.common.config.SslConfigs;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.time.Duration;
import java.util.Properties;

public class KafkaDemoJob {

    public static void main(String[] args) throws Exception {
        System.setProperty("javax.net.ssl.trustStore", "D:\\Downloads\\kafka.client.truststore.jks");
        System.setProperty("javax.net.ssl.trustStorePassword", "KafkaOnsClient");
        StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();

        Properties props = new Properties();
        // props.put(SaslConfigs.SASL_JAAS_CONFIG, "org.apache.kafka.common.security.scram.ScramLoginModule required username=\"alikafka_pre-cn-tl32fsx4l00x\" password=\"VOEdhZLjOrL76lrl5bqPtydtoEkbs0Ny\";");
        props.put(SaslConfigs.SASL_JAAS_CONFIG, "org.apache.kafka.common.security.plain.PlainLoginModule required username=\"alikafka_pre-cn-tl32fsx4l00x\" password=\"VOEdhZLjOrL76lrl5bqPtydtoEkbs0Ny\";");
        props.put(SslConfigs.SSL_TRUSTSTORE_LOCATION_CONFIG, "D:\\Downloads\\kafka.client.truststore.jks");
        props.put(SslConfigs.SSL_TRUSTSTORE_PASSWORD_CONFIG, "KafkaOnsClient");
        props.put(CommonClientConfigs.SECURITY_PROTOCOL_CONFIG, "SASL_SSL");
        props.put(SaslConfigs.SASL_MECHANISM, "PLAIN");
        props.put(SslConfigs.SSL_ENDPOINT_IDENTIFICATION_ALGORITHM_CONFIG, "");
        KafkaSource<String> kafkaSource = KafkaSource.<String>builder()
                .setBootstrapServers("114.55.59.94:9093,112.124.33.132:9093")
                .setTopics("ad_cost_topic")
                .setGroupId("ad_cost_group")
                .setProperties(props)
                .setStartingOffsets(OffsetsInitializer.earliest())
                .setValueOnlyDeserializer(new SimpleStringSchema())
                .build();

        /*DataStreamSource<String> in = env.fromSource(kafkaSource, WatermarkStrategy.noWatermarks(), "Kafka Source");

        SingleOutputStreamOperator<String> adInfo = in.filter(StringUtils::isNotBlank).map(str -> {
            File file = new File("C:\\Users\\hi\\Desktop\\temp\\ff.txt");
            if (!file.exists()) {
                file.createNewFile();
            }
            FileWriter writer = new FileWriter(file, true);
            writer.write(str + "\r\n");
            writer.close();
            return str;
        });*/

        DataStreamSource<String> in = env.fromSource(kafkaSource, WatermarkStrategy.noWatermarks(), "adDataOfMinuteSource_kafka");

        // 广告流
        SingleOutputStreamOperator<AdInfo> adStream = in.filter(StringUtils::isNotBlank)
                .map(str -> JsonUtil.toObj(str, AdDataOfHourDTO.class))
                .map(dto -> {
                    HourlyReportsGetListStruct struct = dto.getHourlyReportsGetListStruct();
                    StatInfo statInfo = new StatInfo();
                    BeanUtils.copyProperties(struct, statInfo);
                    AdInfo adInfo = new AdInfo();
                    adInfo.setStatTime(dto.getCreateTime());
                    adInfo.setStatDay(DateUtil.milliToLocalDate(dto.getCreateTime()));
                    adInfo.setHour(dto.getHourlyReportsGetListStruct().getHour().intValue());
                    adInfo.setAccountId(dto.getAccountId());
                    adInfo.setCampaignId(struct.getCampaignId());
                    adInfo.setAgencyAccountId(struct.getAccountId());
                    adInfo.setWechatAccountId(struct.getWechatAccountId());
                    adInfo.setWechatAgencyId(struct.getWechatAgencyId());
                    adInfo.setAdgroupId(struct.getAdgroupId());
                    adInfo.setAdId(struct.getAdId());
                    adInfo.setStatInfo(statInfo);
                    return adInfo;
                })
                // 打水印，延迟 12分钟，同时指定时间流
                .assignTimestampsAndWatermarks(WatermarkStrategy.<AdInfo>forBoundedOutOfOrderness(Duration.ofMinutes(12L))
                        .withTimestampAssigner((SerializableTimestampAssigner<AdInfo>) (adInfo, l) -> adInfo.getStatTime())
                );

        adStream.keyBy((KeySelector<AdInfo, Long>) AdInfo::getAdId)
                .window(TumblingEventTimeWindows.of(Time.minutes(5L)))
                .apply(new WindowFunction<AdInfo, AdInfo, Long, TimeWindow>() {
                    @Override
                    public void apply(Long key, TimeWindow timeWindow, Iterable<AdInfo> iterable, Collector<AdInfo> collector) throws Exception {
                        for (AdInfo adInfo : iterable) {
                            collector.collect(adInfo);
                        }
                    }
                });

        SingleOutputStreamOperator<PlanInfo> planStream = adStream.keyBy((KeySelector<AdInfo, Long>) AdInfo::getCampaignId)
                .map()
                .window(TumblingEventTimeWindows.of(Time.minutes(5L)))
                .reduce(new PlanOfHourReduceFunction()).map();
        planStream.print();
        // planStatStream.addSink(new PrintSink<>()).name("msg-print");
        env.execute();
    }

    public static final class PlanOfHourReduceFunction implements ReduceFunction<AdInfo> {
        @Override
        public AdInfo reduce(AdInfo oldValue, AdInfo newValue) {
            HourlyReportsGetListStruct oldStruct = oldValue.getHourlyReportsGetListStruct();
            HourlyReportsGetListStruct newStruct = newValue.getHourlyReportsGetListStruct();
            HourlyReportsGetListStruct struct = new HourlyReportsGetListStruct();
            org.springframework.beans.BeanUtils.copyProperties(newStruct, struct);
            struct.setCostDeviationRate(NumberUtil.add(oldStruct.getCostDeviationRate(), struct.getCostDeviationRate()));
            struct.setCost(NumberUtil.add(oldStruct.getCost(), struct.getCost()));
            struct.setCompensationAmount(NumberUtil.add(oldStruct.getCompensationAmount(), struct.getCompensationAmount()));
            struct.setViewCount(NumberUtil.add(oldStruct.getViewCount(), struct.getViewCount()));

            AdDataOfHourDTO result = new AdDataOfHourDTO();
            result.setCreateTime(newValue.getCreateTime());
            result.setAccountId(newValue.getAccountId());
            result.setChannel(newValue.getChannel());
            result.setHourlyReportsGetListStruct(struct);
            return result;
        }
    }

    public static class PrintSink<T> implements SinkFunction<T> {
        private static final long serialVersionUID = 1L;
        private static final Logger LOG = LoggerFactory.getLogger(PrintSink.class);

        @Override
        public void invoke(T value) {
            LOG.info(value == null ? "null" : JsonUtil.toString(value));
        }
    }
}
